@@ -370,7 +370,7 @@ def get_request_response( request_headers = { }  , payload = "" ) :
 
     try :
 
-        response = session.post( endpoint_url , data = payload , headers = request_headers , timeout = endpoint_timeout , verify = endpoint_sslverify )
+        response = session.post( endpoint_url , data = payload , headers = request_headers ,timeout = (endpoint_timeout,10) , verify = endpoint_sslverify )
 
     except requests.exceptions.SSLError as e :
         s_log.write( endpoint_url )
@@ -385,7 +385,12 @@ def get_request_response( request_headers = { }  , payload = "" ) :
         s_log.write( "ERROR Maybe no omnibus..?" )
         s_log.write( endpoint_url )
         s_log.write( request_headers )
+        s_config.set_key( "halt" , True ) 
         return( False )  
+    except requests.exceptions.ReadTimeout as e :
+        s_log.write( endpoint_url)
+        s_log.write( "ERROR requests.exceptions.ReadTimeout:" + str( e ) )
+        return( False )    
 
     #####################################################################
 
