@@ -25,32 +25,72 @@ import shutil
 import gzip
 import inspect
 
+from datetime import datetime, timedelta
+
+import datetime as _dt
+
 from subprocess import Popen, PIPE
 
 from pathlib import Path
 from traceback import format_exception
 
+import urllib.parse
+
 from . import s_log , s_config , s_net , s_db
 
 #####################################################################
 
-class NumpyEncoder( json.JSONEncoder ) :
+
+#class JsonEncoder( json.JSONEncoder ) :
+#    def default( self , z ) :
+#        if isinstance( z , datetime.date ) :
+#            return ( str( z ) )
+#        else :
+#            return super( ).default( z )
+
+class JsonEncoder( json.JSONEncoder ) :
 
     def default( self , obj ) :
-        return  json.JSONEncoder.default( self , obj )
         #if isinstance( obj , numpy.integer ) :
         #    return int( obj )
         #elif isinstance( obj, numpy.floating ) :
         #    return float( obj)
         #elif isinstance( obj, numpy.ndarray ) :
         #    return obj.tolist( )
-        #return json.JSONEncoder.default( self , obj )
+        if isinstance( obj , _dt.date ) :
+            return str( obj ) 
+        return json.JSONEncoder.default( self , obj )
 
 def json_encode( data ) :
-    return json.dumps( data , cls = NumpyEncoder )
+    return json.dumps( data , cls = JsonEncoder )
+
+
+
+
 
 #####################################################################
 
+
+# FIXME TODO aaarrggghhh!!!
+def time_getisoformat( ) :
+
+    dt = datetime.now( datetime.now( ).astimezone( ).tzinfo ).isoformat( )
+
+    return( dt )
+
+
+#>>> now = datetime.now()
+#>>> now
+#datetime.datetime(2021, 11, 29, 11, 55, 27, 172480)
+#>>> now.tzname()
+#>>> from dateutil import tz
+#>>> now = datetime.now(tz=tz.tzlocal())
+#>>> now
+#datetime.datetime(2021, 11, 29, 11, 56, 21, 316679, tzinfo=tzlocal())
+#>>> now.tzname()
+
+
+
 def pickle_load( fn ) :
     if( os.path.isfile( fn ) ) :
         s_log.write_msg( "FOUND " + fn )
@@ -148,17 +188,16 @@ def heartbeat( t = 600 ) :
     heartbeat_last = uptime( )
     return( True )
 
-def ready( ) :
 
-    # FIXME TODO move this out to function
+def resetup_ifconfigchanged( ) :
     config = s_config.get_config( )
     if "/config/init/path_time" in config :
         cd = config[ "/config/cwd" ]
         cip = config[ "/config/init/path" ]
         cipt = config[ "/config/init/path_time" ]
 
-        cfp=cip
-        if not os.path.isabs(cip):
+        cfp = cip
+        if not os.path.isabs( cip ) :
             cfp = cd + "/" + cip
 
         if os.path.isfile( cfp ) :
@@ -167,10 +206,13 @@ def ready( ) :
             if( os.path.getmtime( cfp ) != cipt ) :
                 s_config.setup( )
                 s_db.setup( )
-                s_log.write( "RELOADED " )
+                s_log.write( "resetup_ifconfigchanged True" )
                 #s_log.write( "CONFIG RELOADED " + cfp + str( cipt ) )
 
-    ####################################################################
+
+def ready( ) :
+
+    resetup_ifconfigchanged( )
 
     if( not s_config.isready( ) ) :
         return( False )
@@ -180,6 +222,8 @@ def ready( ) :
 
     return( True )
 
+
+
 ####################################################################
 
 def gen_workingdirectory( ) :
@@ -216,7 +260,7 @@ def proc_setup( ) :
     s_db.files_defmeta_write( )
     ################
 
-    s_db.setup( ) 
+    #s_db.setup( ) 
 
     return( True )
 
@@ -238,23 +282,11 @@ def proc_trackcurrentfiles( ) :
 
 def proc_run( ) :
 
-    pargs = ""
+    if not os.path.isfile( "_sapi.py" ) :
+        s_log.write( "False isfile _sapi.py" )
+        return( False )
 
-    # FIXME TODO filename should be coming from config
-    process = Popen( [ "bash" , "_sapiserver_entrypoint_.sh" , pargs ] , stdout = PIPE , stderr = PIPE )
-
-    stdout , stderr = process.communicate( )
-
-    #print(os.getcwd())
-
-    if stdout :
-        stdout = str( stdout , "utf-8" )
-        if( stdout.strip( ) != "" ) :
-            s_db.stdio_create( 1 , stdout )
-    if stderr :
-        stderr = str( stderr , "utf-8" )
-        if( stderr.strip( ) != "" ) :
-            s_db.stdio_create( 2 , stderr )
+    return( True )
 
 def proc_storage( ) :
 
@@ -272,10 +304,13 @@ def proc_storage( ) :
             file_extension = Path( fname ).suffix
 
             # FIXME TODO extensions (list of) should be coming from config
-            if(file_extension==".pyc"): continue
+            # anything starting with ".*"
+            if( file_extension == ".pyc" ) : continue
 
-            if( not s_db.files_create( fname ) ) :
-                s_log.write( "ERROR files_create " + fname )
+            #s_log.write(fname)
+
+            if( not s_db.files_update( fname ) ) :
+                s_log.write( "ERROR files_update " + fname )
             #with open(fname) as myfile:
             #print(myfile.read())
 
@@ -295,6 +330,7 @@ def decompress( d ) :
     blob = gzip.decompress( d )
     return( blob )
 
+####################################################################
 
 def parent_path( ) :
 
@@ -328,6 +364,43 @@ def parent_path( ) :
     #s_log.write(scr_dict)
 
     return scr_dict["dir"]
+
+################
+
+def container_appenvstatic_get( ) :
+    ENV_STATIC = { }
+    if "_CONTAINER_LOCAL_APPENVSTATIC" in os.environ :
+        ENV_STATIC = dict( urllib.parse.parse_qsl( os.environ[ "_CONTAINER_LOCAL_APPENVSTATIC" ] ) )
+    return( ENV_STATIC )
     
 
+def datetime_iso( ) :
+    return( datetime.now( datetime.now( ).astimezone( ).tzinfo ).isoformat( ) )
+
+
+def url_parse( u ) :
+    return(urllib.parse.urlparse(u))
+
+
+def sleep( t = 1 ) :
+    time.sleep(t) 
+
+
+def datetime_now_db( td_minutes = 0 ) :
+
+    d = datetime.now( ) + timedelta(minutes=td_minutes)
+
+    return(d.strftime('%Y-%m-%d %H:%M:%S'))
+
+def datetime_timedelta_db( dbdtstr , td_minutes ) :
+    d = datetime.strptime(dbdtstr, '%Y-%m-%d %H:%M:%S') + timedelta(minutes=td_minutes)
+    return(d.strftime('%Y-%m-%d %H:%M:%S'))
+
+def datetime_inrange_db( dbdt1 , dbdt2 , td_minutes ) :
+    d1 = datetime.strptime(dbdt1, '%Y-%m-%d %H:%M:%S')  
+    d2 = datetime.strptime(dbdt2, '%Y-%m-%d %H:%M:%S') 
+
+    dtdiff=d2-d1
+    dtdiff_minutes=int(dtdiff.total_seconds()/60)
+    return(dtdiff_minutes<=td_minutes)
 
